@@ -8,7 +8,8 @@
 #' @param prefix (character) optional (species-) prefix for entries in '...', will be only considered if '...' has no names
 #' @param sep (character) concatenation symbol
 #' @param silent (logical) suppress messages
-#' @param callFrom (character) allows easier tracking of message(s) produced
+#' @param debug (logical) display additional messages for debugging
+#' @param callFrom (character) allow easier tracking of message(s) produced
 #' @return This function returns a list with $byPep as list of logical matrixes for each peptide (as line) and unique/shared/etc for each species; $byProt as list of matrixes with count data per proten (as line) for each species; $tab with simple summary-type count data   
 #' @seealso  \code{\link{readFasta2}} and/or \code{cleave-methods} in package \href{https://bioconductor.org/packages/release/bioc/html/cleaver.html}{cleaver}    
 #' @examples
@@ -23,15 +24,18 @@
 #' lapply(mi2$byProt,head)
 #' mi2$tab
 #' @export
-countNoOfCommonPeptides <- function(...,prefix=c("Hs","Sc","Ec"),sep="_",silent=FALSE,callFrom=NULL) {
+countNoOfCommonPeptides <- function(..., prefix=c("Hs","Sc","Ec"), sep="_", silent=FALSE, debug=FALSE, callFrom=NULL) {
   ## compare in-silico digested proteomes for unique and shared peptides, counts per protein
   ## .. input must be lists of proteins wther their respective peptides
   fxNa <- wrMisc::.composeCallName(callFrom,newNa="countNoOfCommonPeptides")
+  if(!isTRUE(silent)) silent <- FALSE
+  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
+
   inp <- list(...)
   chInp <- c("prefix","sep","silent","callFrom") %in% names(inp)
   if(any(chInp)) inp <- inp[which(!chInp)]
-  if(length(inp) <2) stop("not sufficient input elements - nothing to do !")
-  chSep <- sapply(inp,function(x) length(grep(sep,names(x))) >0)
+  if(length(inp) <2) stop("Not sufficient input elements - nothing to do !")
+  chSep <- sapply(inp, function(x) length(grep(sep,names(x))) >0)
   if(any(chSep)) message("Trouble ahead : Separator 'sep' also appears in sequence  names !!")
   ## main
   chN <- names(inp)
@@ -42,16 +46,16 @@ countNoOfCommonPeptides <- function(...,prefix=c("Hs","Sc","Ec"),sep="_",silent=
   .countFra <- function(x) paste(1:x,sep,rep(x,x),sep="")  
   .firstOfRep <- function(x) duplicated(x,fromLast=TRUE) & !duplicated(x,fromLast=FALSE)          # find first of replicated (mark as T)
   names(seqs) <- paste(rep(prefix,nBySet2),sep,unlist(lapply(unlist(nBySet,use.names=FALSE),.countFra)),sep, rep(unlist(lapply(inp,names)),unlist(nBySet)),sep="")
-  seqAnn <- cbind(species=rep(prefix,nBySet2), pepNo=unlist(lapply(unlist(nBySet),function(x) cbind(1:x))),
-    pepTot=unlist(lapply(unlist(nBySet),function(x) cbind(rep(x,x)))), protID=rep(unlist(lapply(inp,names)),unlist(nBySet)))
+  seqAnn <- cbind(species=rep(prefix,nBySet2), pepNo=unlist(lapply(unlist(nBySet), function(x) cbind(1:x))),
+    pepTot=unlist(lapply(unlist(nBySet), function(x) cbind(rep(x,x)))), protID=rep(unlist(lapply(inp,names)),unlist(nBySet)))
   ## make matrix with prot names & number if pep ? (no need for strsplit later - alternative to concatenate as names in seqs)
   staSto <- cumsum(nBySet2)
-  staSto <- cbind(sta=c(1,staSto[-length(staSto)]+1),stop=staSto)
-  out <- list(byPep=list(),byProt=list(),tab=list())
+  staSto <- cbind(sta=c(1,staSto[-length(staSto)] +1), stop=staSto)
+  out <- list(byPep=list(), byProt=list(), tab=list())
   matColNa <- paste(rep(c("uni","shared","red"),2), rep(c("IntrA","InteR"),each=3),sep="")
   matColNa <- c("uni0","shared0","red0",  "unique","sharedInteR","sharedIntrA","redInteR","redIntraA")
   for(i in 1:length(inp)) {
-    mat <- matrix(FALSE,nrow=nBySet2[i],ncol=length(matColNa),dimnames=list(unlist(inp[[i]],use.names=FALSE),matColNa))
+    mat <- matrix(FALSE, nrow=nBySet2[i], ncol=length(matColNa), dimnames=list(unlist(inp[[i]],use.names=FALSE), matColNa))
     frL <- duplicated(seqs[staSto[i,1]:staSto[i,2]],fromLast=TRUE)
     frB <- duplicated(seqs[staSto[i,1]:staSto[i,2]],fromLast=FALSE)
     mat[,1:3] <- matrix(c(!frL & !frB, frL & !frB, frB),ncol=3)
@@ -74,9 +78,9 @@ countNoOfCommonPeptides <- function(...,prefix=c("Hs","Sc","Ec"),sep="_",silent=
     names(out$byPep)[i] <- prefix[i] 
     ## exploit by prot
     tmp <- matrix(unlist(by(mat, rep(names(inp[[i]]), nBySet[[i]]), function(x) colSums(as.matrix(x),na.rm=TRUE))),
-      ncol=ncol(mat), byrow=TRUE, dimnames=list(names(inp[[i]]),colnames(mat)))
+      ncol=ncol(mat), byrow=TRUE, dimnames=list(names(inp[[i]]), colnames(mat)))
     out[["byProt"]][[i]] <- cbind(nPep=nBySet[[i]], matrix(unlist(by(mat,rep(names(inp[[i]]), nBySet[[i]]), function(x) colSums(as.matrix(x),na.rm=TRUE))),
-      ncol=ncol(mat), byrow=TRUE, dimnames=list(names(inp[[i]]),colnames(mat))) )                            # cut in list of matrixes
+      ncol=ncol(mat), byrow=TRUE, dimnames=list(names(inp[[i]]), colnames(mat))) )                            # cut in list of matrixes
     names(out[["byProt"]])[[i]] <- prefix[i] 
     ## summarize
     supl <- c(
@@ -91,8 +95,8 @@ countNoOfCommonPeptides <- function(...,prefix=c("Hs","Sc","Ec"),sep="_",silent=
       min2speIntra=sum(out[["byProt"]][[i]][,"uni0"] >1),                                         # (min 2 pep &) min 2 specif intra
       min2speIner=sum(out[["byProt"]][[i]][,"unique"] >1),                                        # (min 2 pep &) min 2 specif inter
       nLost2pepSpec=sum(out[["byProt"]][[i]][,"uni0"] >1 & out[["byProt"]][[i]][,"unique"] <2) )   # min 2 pep as single species but when mult species combined less than 2 pep
-    out$tab <- if(length(out$tab) <1) as.matrix(c(nProt=length(inp[[i]]),nTotPep=nBySet2[[i]],nPepSpec=sum(mat[,"uni0"])+sum(mat[,"shared0"]), 
-      colSums(mat),supl)) else cbind(out$tab,c(nProt=length(inp[[i]]),nTotPep=nBySet2[[i]],nPepSpec=sum(mat[,"uni0"])+sum(mat[,"shared0"]),colSums(mat),supl))
+    out$tab <- if(length(out$tab) <1) as.matrix(c(nProt=length(inp[[i]]),nTotPep=nBySet2[[i]], nPepSpec=sum(mat[,"uni0"]) +sum(mat[,"shared0"]), 
+      colSums(mat),supl)) else cbind(out$tab,c(nProt=length(inp[[i]]), nTotPep=nBySet2[[i]], nPepSpec=sum(mat[,"uni0"]) +sum(mat[,"shared0"]), colSums(mat),supl))
     colnames(out$tab)[ncol(out$tab)] <- prefix[i]    
     }
   out } 
